"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withRetryLoop = exports.pollUntil = exports.asyncWithTimeout = exports.triggerPromise = exports.wait = void 0;

var R = _interopRequireWildcard(require("ramda"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var wait = function wait(delay) {
  return new Promise(function (r) {
    return setTimeout(r, delay);
  });
};

exports.wait = wait;

var triggerPromise = function triggerPromise() {
  var resolve;
  var reject;
  var promise = new Promise(function (res, rej) {
    resolve = res;
    reject = rej;
  });
  return [promise, resolve, reject];
};

exports.triggerPromise = triggerPromise;

var asyncWithTimeout = function asyncWithTimeout(promise) {
  var timeout = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 3000;
  var timeoutResolve;
  var timeoutPromise = new Promise(function (resolve, reject) {
    timeoutResolve = resolve;
    setTimeout(function () {
      return reject(new Error('asyncWithTimeout: Timeout'));
    }, timeout);
  });
  Promise.race([promise["finally"](timeoutResolve), timeoutPromise]);
};

exports.asyncWithTimeout = asyncWithTimeout;
var pollUntil = R.curry(function () {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$interval = _ref.interval,
      interval = _ref$interval === void 0 ? 10 : _ref$interval,
      _ref$ignoreReject = _ref.ignoreReject,
      ignoreReject = _ref$ignoreReject === void 0 ? false : _ref$ignoreReject;

  var compareFn = arguments.length > 1 ? arguments[1] : undefined;
  var fn = arguments.length > 2 ? arguments[2] : undefined;

  var inner =
  /*#__PURE__*/
  function () {
    var _ref2 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee() {
      var result, threw, compareResult, done;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              threw = false;
              _context.prev = 1;
              _context.next = 4;
              return fn();

            case 4:
              result = _context.sent;
              _context.next = 12;
              break;

            case 7:
              _context.prev = 7;
              _context.t0 = _context["catch"](1);

              if (ignoreReject) {
                _context.next = 11;
                break;
              }

              throw _context.t0;

            case 11:
              threw = true;

            case 12:
              _context.next = 14;
              return compareFn(result);

            case 14:
              compareResult = _context.sent;
              done = !threw && compareResult;

              if (!done) {
                _context.next = 18;
                break;
              }

              return _context.abrupt("return", result);

            case 18:
              return _context.abrupt("return", wait(interval).then(inner));

            case 19:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, null, [[1, 7]]);
    }));

    return function inner() {
      return _ref2.apply(this, arguments);
    };
  }();

  return inner();
});
exports.pollUntil = pollUntil;

var withRetryLoop = function withRetryLoop(_ref3) {
  var onRetry = _ref3.onRetry,
      _ref3$delay = _ref3.delay,
      delay = _ref3$delay === void 0 ? 1 : _ref3$delay,
      _ref3$delta = _ref3.delta,
      delta = _ref3$delta === void 0 ? 2 : _ref3$delta,
      _ref3$maxDelay = _ref3.maxDelay,
      maxDelay = _ref3$maxDelay === void 0 ? 60 : _ref3$maxDelay;
  return function (fn) {
    return function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return fn.apply(void 0, args)["catch"](function (err) {
        if (onRetry) onRetry(err, delay);
        return wait(delay).then(function () {
          return withRetryLoop(fn, {
            onRetry: onRetry,
            delay: Math.min(delay + delta, maxDelay),
            delta: delta,
            maxDelay: maxDelay
          }).apply(void 0, args);
        });
      });
    };
  };
};

exports.withRetryLoop = withRetryLoop;